/**
  ******************************************************************************
  * @file    LcdHal.c
  * @author  MCD Application Team
  * @version V2.0.0
  * @date    11-July-2011
  * @brief   This file contains all the LCD functions whose
  *          implementation depends on the LCD Type used in your Application.
  *          You only need to change these functions implementations
  *          in order to reuse this code with other LCD
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include <stdlib.h>
#include <string.h>
#include "LcdHal.h"
#include "gl_fonts.h"




/** @addtogroup Embedded_GUI_Library
  * @{
  */

/** @defgroup LcdHal 
  * @brief LcdHal main functions
  * @{
  */ 

/* External variables --------------------------------------------------------*/
/* Private typedef -----------------------------------------------------------*/
typedef struct
{
  __IO uint16_t LCD_REG;
  __IO uint16_t LCD_RAM;
} GL_LCD_TypeDef;
/* Private defines -----------------------------------------------------------*/
#ifdef USE_STM3210E_EVAL
  /* Note: LCD /CS is CE4 - Bank 4 of NOR/SRAM Bank 1~4 */
  #define GL_LCD_BASE ((uint32_t)(0x60000000 | 0x0C000000))
#elif USE_STM322xG_EVAL
/* Note: LCD /CS is NE3 - Bank 3 of NOR/SRAM Bank 1~4 */
  #define GL_LCD_BASE ((uint32_t)(0x60000000 | 0x08000000))
#elif USE_STM32100E_EVAL
  /* Note: LCD /CS is CE4 - Bank 4 of NOR/SRAM Bank 1~4 */
  #define GL_LCD_BASE           ((uint32_t)(0x60000000 | 0x0C000000))
#endif

#define GL_LCD                ((__IO GL_LCD_TypeDef *) GL_LCD_BASE)
#define RCC_AHBPeriph_FSMC    ((uint32_t)0x00000100)
/* Private macros ------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* WARNING: If LcdHal functions are used by more Graphic Applications, remember that the
   following static variables will be shared among the several Graphic Objects applications */
/* Global variables to handle the right font */
__IO uint8_t GL_Font = GL_FONT_BIG;
__IO uint8_t GL_FontWidth = GL_FONT_BIG_WIDTH;
__IO uint8_t GL_FontHeight = GL_FONT_BIG_HEIGHT;

uint16_t LCD_Height = 240;
uint16_t LCD_Width  = 320;
LCD_Direction_TypeDef LCD_Direction = _0_degree;

/* LCD Hardware Parameters Structure */
LCD_HW_Parameters_TypeDef pLcdHwParam;  

#if !defined(LCD_ILI9320)
  #define LCD_ILI9320 0x9320
#endif

#if !defined(LCD_SPFD5408)
  #define LCD_SPFD5408 0x5408
#endif

#if !defined(LCDType)
  __IO uint32_t LCDType = LCD_ILI9320;
#else
  extern __IO uint32_t LCDType;
#endif
  
__IO uint16_t  GL_TextColor;
__IO uint16_t  GL_BackColor;

/* Private function prototypes -----------------------------------------------*/
static void BmpBuffer32BitRead(uint32_t* ptr32BitBuffer, uint8_t* ptrBitmap);
/* Private functions ---------------------------------------------------------*/


/**
  * @brief  Create and initialize a new Lcd Hw Parameter structure object
  * @param  None
  * @retval LCD_HW_Parameters_TypeDef* - The created Object pointer
  */
LCD_HW_Parameters_TypeDef* NewLcdHwParamObj ( void )
{
  return &pLcdHwParam;
}

uint16_t GL_BackgroundColor;
uint16_t GL_ForegroundColor;


/**
  * @brief  Displays a maximum of 20 char on the LCD.
  * @param  Line: the Line where to display the character shape.
  * @param  *ptr: pointer to the string to display on LCD.
  * @param  Transparent_Flag: if TRUE the character is printed without changing the background
  * @retval None
  */
void GL_DisplayAdjStringLine(uint16_t Line, uint16_t Column, char *ptr, GL_bool Transparent_Flag)
{
  uint32_t index = 0;
  uint32_t iMaxChar = (Column / (GL_FontWidth - 1));

  /* Send the string character by character on lCD */
  while ((*ptr != 0) & (index < iMaxChar))
  { /* Display one character on LCD */
    GL_PrintChar(Column, Line, *ptr, Transparent_Flag);
    /* Decrement the column position by GL_FontWidth */
    if ( *ptr == 'A' || *ptr == 'G' || *ptr == 'M' || *ptr == 'O' || *ptr == 'Q' || *ptr == 'X' || *ptr == 'm')
      Column -= (GL_FontWidth);
    else
      Column -= (GL_FontWidth - 1);
    /* Point on the next character */
    ptr++;
    /* Increment the character counter */
    index++;
  }
}

/**
  * @brief  Displays one character (16dots width, 24dots height).
  * @param  Line: the Line where to display the character shape.
  *         This parameter can be one of the following values:
  *     @arg  - Linex: where x can be 0..9
  * @param  Column: start column address.
  * @param  Ascii: character ascii code, must be between 0x20 and 0x7E.
  * @param  GL_bool Trasparent_Flag, if TRUE it does not print the GL_BackColor
  * @retval None
  */

/**
  * @brief  Disables LCD window mode.
  * @param  None
  * @retval None
  */
void GL_DisableWindowMode(void)
{
  LCD_SetDisplayWindow(LCD_Width - 1, LCD_Height - 1, LCD_Height, LCD_Width);
  /* Set GRAM write direction and BGR = 1 */
  /* I/D=10 (Horizontal : decrement, Vertical : increment) */
  /* AM=0 (address is updated in horizontal writing direction) */
  LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x20);
}



/*(0,0)
 *  x----->
 *  |   columns
 *  |
 *  V lines
 */
/**
  * @brief  Prints a string of characters on LCD
  * @param  Line:   specifies the line number of top left corner of the string (depends on orientation).
  * @param  Column: specifies the column number of top left corner of the string (depends on orientation).
  * @param  *ptr:   the string of characters to be printed.
  * @retval None
  */
void GL_PrintString(uint16_t Line, uint16_t Column, uint8_t *ptr)
{
  uint8_t counter = 0;
  uint8_t max_char_line = 0;
  uint16_t column = 0;
  uint16_t line = 0;
  //uint16_t temp;

  if (LCD_Direction == _0_degree)
  {
    max_char_line = (LCD_Width / GL_FontWidth);
    column = Column;
    line = Line;
  }
  else if (LCD_Direction == _90_degree)
  {
    max_char_line = (LCD_Height / GL_FontWidth);
    column = Line;
    line = Column;
  }
  else if (LCD_Direction == _180_degree)
  {
    max_char_line = (LCD_Width / GL_FontWidth);
    column = Column;
    line = LCD_Height - Line;
  }
  else if (LCD_Direction == _270_degree)
  {
    max_char_line = (LCD_Height / GL_FontWidth);
    column = Column;
    line = LCD_Width - Line;
  }

  /* Send the string character by character on LCD */
  while ((*ptr != 0) & (counter < max_char_line))
  {
    /* Display one character on LCD */
    GL_PrintChar(line, column, *ptr, GL_FALSE);

    if (LCD_Direction == _0_degree)
    {
      /* Increment the column position for next character */
      column += GL_FontWidth;
    }
    else if (LCD_Direction == _90_degree)
    {
      /* Increment the column position by 16 */
      line -= GL_FontWidth;
    }
    else if (LCD_Direction == _180_degree)
    {
      /* Decrement the column position by 16 */
      column -= GL_FontWidth;
    }
    else if (LCD_Direction == _270_degree)
    {
      /* Increment the column position by 16 */
      line += GL_FontWidth;
    }

    /* Point on the next character */
    ptr++;
    /* Increment the character counter */
    counter++;
  }
}






/**
  * @brief  Sets a display window
  * @param  Xpos:  specifies the X bottom left position.
  * @param  Ypos:  specifies the Y bottom left position.
  * @param  Height: display window height.
  * @param  Width:  display window width.
  * @retval None
  */
void GL_SetDisplayWindow(uint16_t Xpos, uint8_t Ypos, uint8_t Height, uint16_t Width)
{
  LCD_WriteReg(SSD2119_H_RAM_START_REG, (Xpos - Width + 1) );
  LCD_WriteReg(SSD2119_H_RAM_END_REG, Xpos );

  LCD_WriteReg(SSD2119_V_RAM_POS_REG, (Ypos - Height + 1) | (Ypos << 8) );

  LCD_SetCursor(Xpos, Ypos);
}

/**
  * @brief  Changes drawing mode
  * @param  Direction: the drawing direction.
  *         This parameter can be one of the following values:
  *     @arg  _0_degree
  *     @arg  _90_degree
  *     @arg  _180_degree
  *     @arg  _270_degree
  * @retval None
  */
void GL_ChangeDirection(LCD_Direction_TypeDef Direction)
{
  LCD_Direction = Direction;

  if (LCD_Direction == _0_degree)
  {
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=10 (Horizontal : decrement, Vertical : increment) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x20);
  }
  else if (LCD_Direction == _90_degree)
  {
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=00 (Horizontal : decrement, Vertical : decrement) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x08);
  }
  else if (LCD_Direction == _180_degree)
  {
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=01 (Horizontal : increment, Vertical : decrement) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x10);
  }
  else if (LCD_Direction == _270_degree)
  {
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=11 (Horizontal : increment, Vertical : increment) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x38);
  }
}

/**
  * @brief  Clears the whole LCD.
  * @param  Color: specifies the Background color code RGB(5-6-5) for the Display.
  * @retval None
  */
void GL_Clear(uint16_t Color)
{
  LCD_Clear(Color);
}

/**
  * @brief  Draws a character on LCD with or without background.
  * @param  Xpos: specifies the top left X position of the character (dependent on orientation; linked to character).
  * @param  Ypos: specifies the top left Y position of the character (dependent on orientation; linked to character).
  * @param  c:    pointer to the character data.
  * @param  Transparent_Flag: if TRUE the character is printed without changing the background.
  * @retval None
  */
void GL_DrawChar(uint16_t Xpos, uint16_t Ypos, const uint16_t* c, GL_bool Transparent_Flag)
{
  uint8_t line_index = 0, pixel_index = 0;
  uint16_t Xaddress = Xpos;
  uint8_t Yaddress = Ypos;

  for (line_index = 0; line_index < GL_FontHeight; line_index++)
  {
    LCD_SetCursor(Xaddress, Yaddress);
    /* Prepare to write GRAM */
    LCD_WriteRAM_Prepare();

    for (pixel_index = 0; pixel_index < GL_FontWidth; pixel_index++)
    {
      /* SmallFonts have bytes in reverse order */
      if (( GL_Font == GL_FONT_BIG   && (((const uint16_t*)c)[line_index] & (1 << pixel_index)) == 0x00) ||
          ( GL_Font == GL_FONT_SMALL && (c[line_index] & (0x80 >> pixel_index)) == 0x00))
      {
        if(Transparent_Flag == GL_FALSE)
        {
          LCD_WriteRAM(GL_BackgroundColor);
        }
        else
        {
          LCD_ReadRAM(); // Dummy read for moving cursor
        }
      }
      else
      {
        LCD_WriteRAM(GL_TextColor);
      }
    }

    if (LCD_Direction == _0_degree)
    {
      Xaddress = Xpos;
      Yaddress++;
    }
    else if (LCD_Direction == _90_degree)
    {
      Xaddress--;
      Yaddress = Ypos;
    }
    else if (LCD_Direction == _180_degree)
    {
      Xaddress = Xpos;
      Yaddress--;
    }
    else if (LCD_Direction == _270_degree)
    {
      Xaddress++;
      Yaddress = Ypos;
    }
  }
}

/**
  * @brief  Displays a circle.
  * @param  Xpos:   specifies the X position of circle center.
  * @param  Ypos:   specifies the Y position of circle center.
  * @param  Radius: the radius size of the circle.
  * @retval None
  */
void GL_DrawCircle(uint16_t Xpos, uint16_t Ypos, uint16_t Radius)
{
  LCD_DrawCircle(Xpos, Ypos, Radius);
}

void GL_DrawFilledRectangle(uint16_t Xpos, uint16_t Ypos, uint16_t Width, uint16_t Height, uint16_t Color){
  LCD_DrawFilledRectangle(Xpos, Ypos, Width, Height, Color);
}

/**
  * @brief  Displays a line.
  * @param  Xpos:      specifies the X position.
  * @param  Ypos:      specifies the Y position.
  * @param  Length:    line length.
  * @param  Direction: line direction.
  *         This parameter can be one of the following values:
  *     @arg  Vertical
  *     @arg  Horizontal
  * @retval None
  */
void GL_DrawLine(uint16_t Xpos, uint16_t Ypos, uint16_t Length, uint8_t Direction)
{
  LCD_DrawLine(Xpos, Ypos, Length, Direction);
}

/**
  * @brief  Displays a line.
  * @param  X1: specifies the X1 position.
  * @param  Y1: specifies the Y1 position.
  * @param  X2: specifies the X2 position.
  * @param  Y2: specifies the Y2 position.
  * @retval None
  */
void GL_DrawLine_2(uint16_t X1, uint16_t Y1, uint16_t X2, uint16_t Y2)
{
  LCD_DrawLine_2(X1, Y1, X2, Y2);
}

/**
  * @brief  Displays a rectangle.
  * @param  Xpos:   specifies the X position.
  * @param  Ypos:   specifies the Y position.
  * @param  Height: rectangle height.
  * @param  Width:  rectangle width.
  * @retval None
  */
void GL_DrawRectangle(uint16_t Xpos, uint16_t Ypos, uint16_t Width, uint16_t Height){
  LCD_DrawRectangle(Xpos, Ypos, Width, Height);
}

/**
  * @brief  Draws a string of characters on LCD.
  * @param  Xpos: specifies the top left X position of the text (independent from orientation; linked to screen).
  * @param  Ypos: specifies the top left Y position of the text (independent from orientation; linked to screen).
  * @param  ptr:  the string of characters to be printed.
  * @param  Transparent_Flag: if TRUE the character is printed without changing the background.
  * @retval None
  */
void GL_DrawString(uint16_t Xpos, uint8_t Ypos, uint8_t *ptr, GL_bool Transparent_Flag)
{
  uint16_t Xaddress = Xpos;
  uint8_t Yaddress = Ypos;

  if (LCD_Direction == _90_degree)
  {
     Yaddress = Ypos + strlen((char*)ptr)*GL_FontWidth - 1;
  }
  else if (LCD_Direction == _180_degree)
  {
     Xaddress = Xpos - strlen((char*)ptr)*GL_FontWidth + 1;
     Yaddress = Ypos + GL_FontHeight - 1;
  }
  else if (LCD_Direction == _270_degree)
  {
     Xaddress = Xpos - GL_FontHeight + 1;
  }

  while((*ptr != 0))
  {
    GL_PrintChar(Xaddress, Yaddress, *ptr, Transparent_Flag);

    if (LCD_Direction == _0_degree)
    {
       Xaddress -= GL_FontWidth;
    }
    else if (LCD_Direction == _90_degree)
    {
       Yaddress -= GL_FontWidth;
    }
    else if (LCD_Direction == _180_degree)
    {
       Xaddress += GL_FontWidth;
    }
    else if (LCD_Direction == _270_degree)
    {
       Yaddress += GL_FontWidth;
    }

    ptr++;
  }
}

uint16_t GL_GetBackgroundColor()
{
  GL_BackgroundColor = LCD_GetBackgroundColor();
  return GL_BackgroundColor;
}

uint16_t GL_GetForegroundColor()
{
  GL_ForegroundColor = LCD_GetForegroundColor();
  return GL_ForegroundColor;
}

uint16_t GL_GetTextColor()
{
  GL_TextColor = LCD_GetTextColor();
  return GL_TextColor;
}

/**
  * @param  Xpos:  specifies the top left X position of the character (dependent on orientation; linked to character).
  * @param  Ypos:  specifies the top left Y position of the character (dependent on orientation; linked to character).
  * @param  Ascii: the character ASCII code.
  * @param  Transparent_Flag: if TRUE the character is printed without changing the background.
  * @retval None
  */
void GL_PrintChar(uint16_t Xpos, uint16_t Ypos, uint8_t Ascii, GL_bool Transparent_Flag)
{
  Ascii -= 32;

  switch (GL_Font)
  {
    case GL_FONT_BIG:
      GL_DrawChar( Xpos, Ypos, &GL_Font16x24.table[Ascii * GL_FontHeight], Transparent_Flag );
      break;

    case GL_FONT_SMALL:
      GL_DrawChar( Xpos, Ypos, &GL_Font8x12_bold.table[Ascii * GL_FontHeight], Transparent_Flag );
      break;

    default:
      break;
  }
}

/**
  * @brief  Sets the background color.
  * @param  Color: specifies the background color code RGB(5-6-5).
  * @retval None
  */
void GL_SetBackgroundColor(uint16_t Color)
{
  GL_BackgroundColor = Color;
  LCD_SetBackgroundColor(GL_BackgroundColor);
}

/**
  * @brief  Sets the Font (Big or Small).
  * @param  uFont: specifies the Font.
  *         This parameter can be one of the following values:
  *     @arg  GL_FONT_BIG
  *     @arg  GL_FONT_SMALL
  * @retval None
  */
void GL_SetFont(uint8_t uFont)
{
  GL_Font = uFont;

  switch (uFont)
  {
    case GL_FONT_BIG:
      GL_FontWidth  = GL_FONT_BIG_WIDTH;
      GL_FontHeight = GL_FONT_BIG_HEIGHT;
      break;

    case GL_FONT_SMALL:
      GL_FontWidth  = GL_FONT_SMALL_WIDTH;
      GL_FontHeight = GL_FONT_SMALL_HEIGHT;
      break;

    default:
      break;
  } /* End switch */
}

/**
  * @brief  Sets the foreground color.
  * @param  Color: specifies the foreground color code RGB(5-6-5).
  * @retval None
  */
void GL_SetForegroundColor(uint16_t Color)
{
  GL_ForegroundColor = Color;
  LCD_SetForegroundColor(GL_ForegroundColor);
}

/**
  * @brief  Sets the Text color.
  * @param  Color: specifies the Text color code RGB(5-6-5).
  * @param  GL_TextColor: Text color global variable used by GL_DrawChar
  *         and GL_DrawPicture functions.
  * @retval None
  */
void GL_SetTextColor(uint16_t Color)
{
  GL_TextColor = Color;
  LCD_SetTextColor(GL_TextColor);
}

/**
  * @brief  Displays a bitmap picture
  * @param  BmpAddress: Bmp picture address
  * @retval None
  */
void GL_DrawBMP(uint8_t* ptrBitmap)
{
  uint32_t uDataAddr = 0, uBmpSize = 0;
  uint16_t uBmpData;

  BmpBuffer32BitRead(&uBmpSize, ptrBitmap + 2);
  BmpBuffer32BitRead(&uDataAddr, ptrBitmap + 10);

  /* Set GRAM write direction and BGR = 1 */
  /* I/D=00 (Horizontal : decrement, Vertical : decrement) */
  /* AM=1 (address is updated in vertical writing direction) */
  LCD_WriteReg(0x11, 0x6800);
  LCD_WriteRAM_Prepare(); /* Prepare to write GRAM */

  /* Read bitmap data and write them to LCD */
  for (; uDataAddr < uBmpSize; uDataAddr += 2)
  {
    uBmpData = (uint16_t)(*(ptrBitmap + uDataAddr)) + (uint16_t)((*(ptrBitmap + uDataAddr + 1)) << 8);
    LCD_WriteRAM( uBmpData );
  }

  if ( pLcdHwParam.LCD_Connection_Mode == GL_SPI )
    GL_LCD_CtrlLinesWrite(pLcdHwParam.LCD_Ctrl_Port_NCS, pLcdHwParam.LCD_Ctrl_Pin_NCS, GL_HIGH);

  /* Set GRAM write direction and BGR = 1 */
  /* I/D = 01 (Horizontal : increment, Vertical : decrement) */
  /* AM = 1 (address is updated in vertical writing direction) */
  //LCD_WriteReg(0x11, 0x6818);
  LCD_WriteReg(0x11, 0x6820);
}



/**
  * @brief  Switches the backlight either ON or OFF
  * @param  state. This parameter can be one of the following values:
  *     @arg   GL_ON
  *     @arg   GL_OFF
  * @retval None
  */
void GL_BackLightSwitch(uint8_t u8_State)
{
  if (u8_State == GL_OFF)
  { /* Turning OFF the LCD Backlight */
    /* GPIO_ResetBits(LCD_GPIO_DATA_PORT, GPIO_Pin_4); */
    LCD_DisplayOff();
  }
  else if (u8_State == GL_ON)
  { /* Turning ON the LCD Backlight */
    /* GPIO_SetBits(LCD_GPIO_DATA_PORT, GPIO_Pin_4); */
    LCD_DisplayOn();
  }
}

/**
  * @brief  Initializes the LCD.
  * @param  None
  * @retval None
  */
void GL_LCD_Init(void)
{
    /* Setups the LCD */
#if defined(USE_STM3210C_EVAL)
  STM3210C_LCD_Init();  
#elif defined (USE_STM3210B_EVAL)
  STM3210B_LCD_Init();
#elif  defined (USE_STM32100B_EVAL)
  STM32100B_LCD_Init();
#elif defined(USE_STM3210E_EVAL)
  STM3210E_LCD_Init();
#elif defined(USE_STM32100E_EVAL)
  STM32100E_LCD_Init();
#elif defined(USE_STM322xG_EVAL)
  STM322xG_LCD_Init();
#elif defined(USE_STM32L152_EVAL)  
  STM32L152_LCD_Init();
#endif
  STM32f4_Discovery_LCD_Init();
}



/**
  * @brief  Draw one pixel at position given by Xpos, Ypos of color Color.
  *     Three different modes are specified by PixelSpec in order to save time:
  *     1.FirstPixel, MiddlePixel and LasPixel are used for fastened block-writing
  *       to GRAM (for instance: drawing rectangle, horizontal line etc.)
  *     2.SinglePixel is used for drawing stand-alone pixel. (for instance:
  *       drawing circle, character etc.).
  * @param  Xpos: specifies X position
  * @param  Ypos: specifies Y position
  * @param  Color: RGB color of point
  * @param  PixelSpec: specifies Mode of putting pixel.
  *         This parameter can be one of the following values:
  *     @arg  - FirstPixel:  Starting pixel of block-writing sequence.
  *     @arg  - MiddlePixel: Middle-located pixel of block-writing sequence.
  *     @arg  - LasPixel:    Ending pixel of block-writing sequence
  *     @arg  - SinglePixel: Separated pixel.
  * @retval None
  */
void LCD_PutPixel(uint16_t Xpos, uint16_t Ypos, uint16_t Color, uint8_t PixelSpec)
{
  /*Start part of put pixel for first pixel of block and for single one.
  It consists of set the cursor's position and GRAM prepare sequence.*/
  if ((PixelSpec & (FirstPixel | SinglePixel)) != GL_RESET)
  {
    LCD_SetCursor(Xpos, Ypos);
    LCD_WriteRAM_Prepare();
  }

  /*Write pixel's color to GRAM. Obligatory for all modes of PutPixel call.*/
  LCD_WriteRAM(Color);

  if ( pLcdHwParam.LCD_Connection_Mode == GL_SPI)
  {
    /*End part of putting pixel for last pixel of block or single pixel.
    Close usage of display by setting CS high.*/
    if ((PixelSpec &(LastPixel | SinglePixel) ) != GL_RESET)
    {
      if ( pLcdHwParam.LCD_Connection_Mode == GL_SPI )
      {
        GL_LCD_CtrlLinesWrite(pLcdHwParam.LCD_Ctrl_Port_NCS, pLcdHwParam.LCD_Ctrl_Pin_NCS, GL_HIGH);
      }
    }
  }
}

/**
  * @brief  Get color of pixel located at appropriate position
  * @param  Xpos: specifies X position
  * @param  Ypos: specifies Y position
  * @retval uint16_t - RGB color of required pixel.
  */
uint16_t LCD_GetPixel(uint16_t Xpos, uint16_t Ypos)
{
  uint16_t tmpColor = 0, tmpRed = 0, tmpBlue = 0;

  /*Read the color of given pixel*/
  LCD_SetCursor(Xpos, Ypos);
  if ( pLcdHwParam.LCD_Connection_Mode == GL_SPI)
  {
    tmpColor = LCD_ReadReg(0x34);
  }
  else
  {
    tmpColor = GL_LCD_ReadRAM();
  }

  /*Swap the R and B color channels*/
  tmpRed = (tmpColor & Blue) << 11;
  tmpBlue = (tmpColor & Red) >> 11;
  tmpColor &= ~(Red | Blue);
  tmpColor |= (tmpRed | tmpBlue);

  return tmpColor;
}

/**
  * @brief  Copy 4 bytes from bitmap array to 32Bit buffer
  * @param  ptrBitmap - Bitmap pointer
  * @param  ptr32BitBuffer - 32Bit buffer to fill
  * @retval None
  */
void BmpBuffer32BitRead(uint32_t* ptr32BitBuffer, uint8_t* ptrBitmap)
{
  *ptr32BitBuffer = 0;
  *ptr32BitBuffer = (*ptrBitmap);
  *ptr32BitBuffer += (*(ptrBitmap + 1)) << 8;
  *ptr32BitBuffer += (*(ptrBitmap + 2)) << 16;
  *ptr32BitBuffer += (*(ptrBitmap + 3)) << 24;
}

/**
  * @brief  Sets or reset LCD control lines.
  * @param  GPIOx: where x can be B or D to select the GPIO peripheral.
  * @param  CtrlPins: the Control line.
  *         This parameter can be one of the following values:
  *     @arg  - LCD_CtrlPin_NCS: Chip Select pin 
  *     @arg  - CtrlPin_NWR: Read/Write Selection pin
  *     @arg  - CtrlPin_RS: Register/RAM Selection pin
  * @param  BitVal: specifies the value to be written to the selected bit.
  *         This parameter can be one of the following values:
  *     @arg  - GL_LOW: to clear the port pin
  *     @arg  - GL_HIGH: to set the port pin
  * @retval None
  */
void GL_LCD_CtrlLinesWrite(GPIO_TypeDef* GPIOx, uint16_t CtrlPins, GL_SignalActionType BitVal)
{
#if defined(USE_STM3210C_EVAL) || defined (USE_STM3210B_EVAL) ||\
  defined (USE_STM32100B_EVAL) || defined(USE_STM32L152_EVAL)
  /* Set or Reset the control line */
  LCD_CtrlLinesWrite(GPIOx, CtrlPins, (BitAction)BitVal);
#endif
}

/**
  * @brief  Reads the LCD RAM.
  * @param  None
  * @retval uint16_t - LCD RAM Value.
  */
uint16_t GL_LCD_ReadRAM(void)
{
  __IO uint16_t tmp = 0;

#if defined(GL_LCD_BASE)
  /* Write 16-bit Index (then Read Reg) */
  GL_LCD->LCD_REG = R34; /* Select GRAM Reg */
  /* Read 16-bit Reg */
  tmp = GL_LCD->LCD_RAM;
  tmp = GL_LCD->LCD_RAM; /* This line must be removed to work with some LCD controller */
#else
  if( pLcdHwParam.LCD_Connection_Mode == GL_SPI)
  {
    tmp = LCD_ReadReg(0x34);
  }
#endif

  return tmp;
}

//#if defined (USE_STM3210E_EVAL) || defined (USE_STM32100E_EVAL) || defined (USE_STM322xG_EVAL)
/**
  * @brief  Writes 1 word to the LCD RAM.
  * @param  RGB_Code: the pixel color in RGB mode (5-6-5).
  * @retval None
  */
void LCD_WriteRAMWord(uint16_t RGB_Code)
{
  LCD_WriteRAM_Prepare();
  LCD_WriteRAM(RGB_Code);
}
//#endif /* USE_STM3210E_EVAL */














/**
  * @brief  LCD_DrawMonoBMP
  * @param  *Pict:   The pointer to the image
  * @param  Xpos_Init: The X axis position
  * @param  Ypos_Init: The Y axis position
  * @param  Height:    The Height of the image
  * @param  Width:     The Width of the image
  * @retval None
  */
void LCD_DrawMonoBMP(const uint8_t *Pict, uint16_t Xpos_Init, uint16_t Ypos_Init, uint16_t Height, uint16_t Width)
{
  int32_t index = 0, counter = 0;


  if (LCD_Direction == _0_degree)
  {
    LCD_SetDisplayWindow(Xpos_Init, Ypos_Init, Height, Width);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=00 (Horizontal : decrement, Vertical : decrement) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(0x11, 0x6808);
  }
  else if (LCD_Direction == _90_degree)
  {
    LCD_SetDisplayWindow(Xpos_Init + Width - 1, Ypos_Init, Width, Height);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=01 (Horizontal : increment, Vertical : decrement) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(0x11, 0x6810);
  }
  else if (LCD_Direction == _180_degree)
  {
    LCD_SetDisplayWindow(Xpos_Init + Height - 1, Ypos_Init + Width - 1, Height, Width);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=11 (Horizontal : increment, Vertical : increment) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(0x11, 0x6838);
  }
  else if (LCD_Direction == _270_degree)
  {
    LCD_SetDisplayWindow(Xpos_Init, Ypos_Init + Height - 1, Width, Height);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=10 (Horizontal : decrement, Vertical : increment) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(0x11, 0x6820);
  }

  LCD_SetCursor(Xpos_Init, Ypos_Init);

  if ((LCDType == LCD_ILI9320) || (LCDType == LCD_SPFD5408))
  {
    /* Prepare to write GRAM */
    LCD_WriteRAM_Prepare(); 
  }

  for (index = 0; index < (Height*Width) / 8; index++)
  {
    for (counter = 7; counter >= 0; counter--)
    {
      if ((Pict[index] & (1 << counter)) == 0x00)
      {
        LCD_WriteRAM(GL_BackColor);
      }
      else
      {
        LCD_WriteRAM(GL_TextColor);
      }
    }
  }

  if ((LCDType == LCD_ILI9320) || (LCDType == LCD_SPFD5408))
  {
    if ( pLcdHwParam.LCD_Connection_Mode == GL_SPI )
    {
      GL_LCD_CtrlLinesWrite(pLcdHwParam.LCD_Ctrl_Port_NCS, pLcdHwParam.LCD_Ctrl_Pin_NCS, GL_HIGH);
    }
  }
  GL_ChangeDirection(LCD_Direction);
  GL_SetDisplayWindow(LCD_Height - 1, LCD_Width - 1, LCD_Height, LCD_Width);
}

/**
  * @brief  Fill area with color.
  * @param  maxX: Maximum X coordinate
  * @param  minX: Minimum X coordinate
  * @param  maxY: Maximum Y coordinate
  * @param  minY: Minimum Y coordinate
  * @param  ptrBitmap: pointer to the image
  * @retval None
  */
void LCD_FillArea(uint16_t Xpos_Init, uint16_t Ypos_Init, uint16_t Height, uint16_t Width, uint16_t color)
{
  uint32_t area = 0;
  uint32_t index = 0;

  area =  Width *  Height;
  if (LCD_Direction == _0_degree)
  {
    GL_SetDisplayWindow(Xpos_Init, Ypos_Init, Height, Width);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=00 (Horizontal : decrement, Vertical : decrement) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(0x11, 0x6808);
  }
  else if (LCD_Direction == _90_degree)
  {
    GL_SetDisplayWindow(Xpos_Init + Width - 1, Ypos_Init, Width, Height);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=01 (Horizontal : increment, Vertical : decrement) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(0x11, 0x6810);
  }
  else if (LCD_Direction == _180_degree)
  {
    GL_SetDisplayWindow(Xpos_Init + Height - 1, Ypos_Init + Width - 1, Height, Width);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=11 (Horizontal : increment, Vertical : increment) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(0x11, 0x6838);
  }
  else if (LCD_Direction == _270_degree)
  {
    GL_SetDisplayWindow(Xpos_Init, Ypos_Init + Height - 1, Width, Height);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=10 (Horizontal : decrement, Vertical : increment) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(0x11, 0x6820);
  }

  LCD_SetCursor(Xpos_Init, Ypos_Init);

  if ((LCDType == LCD_ILI9320) || (LCDType == LCD_SPFD5408))
  {
    LCD_WriteRAM_Prepare(); /* Prepare to write GRAM */
  }

  for (index = 0;index < area; index++)
  {
    LCD_WriteRAM(color);
  }

  if ((LCDType == LCD_ILI9320) || (LCDType == LCD_SPFD5408))
  {
    if ( pLcdHwParam.LCD_Connection_Mode == GL_SPI )
    {
      GL_LCD_CtrlLinesWrite(pLcdHwParam.LCD_Ctrl_Port_NCS, pLcdHwParam.LCD_Ctrl_Pin_NCS, GL_HIGH);
    }
  }
  GL_DisableWindowMode();
  GL_ChangeDirection(LCD_Direction);

}

/**
  * @brief  LCD_DrawColorBMP
  * @param  *ptrBitmap:   The pointer to the image
  * @param  Xpos_Init: The X axis position
  * @param  Ypos_Init: The Y axis position
  * @param  Height:    The Height of the image
  * @param  Width:     The Width of the image
  * @retval None
  */
void LCD_DrawColorBMP(uint8_t* ptrBitmap, uint16_t Xpos_Init, uint8_t Ypos_Init, uint8_t Height, uint16_t Width)
{
  uint32_t uDataAddr = 0, uBmpSize = 0;
  uint16_t uBmpData = 0;

  BmpBuffer32BitRead(&uBmpSize, ptrBitmap + 2);  
  BmpBuffer32BitRead(&uDataAddr, ptrBitmap + 10);  

  if (LCD_Direction == _0_degree)
  {
    GL_SetDisplayWindow(Xpos_Init, Ypos_Init, Height, Width);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=00 (Horizontal : decrement, Vertical : decrement) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x0);
  }
  else if (LCD_Direction == _90_degree)
  {
    GL_SetDisplayWindow(Xpos_Init + Height - 1, Ypos_Init, Width, Height);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=01 (Horizontal : increment, Vertical : decrement) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x18);
  }
  else if (LCD_Direction == _180_degree)
  {
    GL_SetDisplayWindow(Xpos_Init + Width - 1, Ypos_Init + Height - 1, Height, Width);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=11 (Horizontal : increment, Vertical : increment) */
    /* AM=0 (address is updated in horizontal writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x30);
  }
  else if (LCD_Direction == _270_degree)
  {
    GL_SetDisplayWindow(Xpos_Init, Ypos_Init + Width - 1, Width, Height);
    /* Set GRAM write direction and BGR = 1 */
    /* I/D=10 (Horizontal : decrement, Vertical : increment) */
    /* AM=1 (address is updated in vertical writing direction) */
    LCD_WriteReg(SSD2119_ENTRY_MODE_REG, 0x6800 | 0x28);
  }

  LCD_SetCursor(Xpos_Init, Ypos_Init);

  if ((LCDType == LCD_ILI9320) || (LCDType == LCD_SPFD5408))
  {
    /* Prepare to write GRAM */
    LCD_WriteRAM_Prepare(); 
  }

  /* Read bitmap data and write them to LCD */
  for (; uDataAddr < uBmpSize; uDataAddr += 2)
  {
    uBmpData = (uint16_t)(*(ptrBitmap + uDataAddr)) + (uint16_t)((*(ptrBitmap + uDataAddr + 1)) << 8);
    LCD_WriteRAM( uBmpData );
  }

  GL_ChangeDirection(LCD_Direction);
  GL_SetDisplayWindow(LCD_Width - 1, LCD_Height - 1, LCD_Height, LCD_Width);
}
















/**
  * @}
  */ 

/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
